/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Fabian 'x3n' Landau
 *   Co-authors:
 *      ...
 *
 */

/**
    @file ParticleProjectile.h
    @brief Implementation of the ParticleProjectile class.
*/

#include "BallProjectile.h"

#include <OgreParticleEmitter.h>
#include "core/CoreIncludes.h"
#include "tools/ParticleInterface.h"
#include "Scene.h"

namespace orxonox
{
    RegisterClass(BallProjectile);

    BallProjectile::BallProjectile(Context* context) : Projectile(context)
    {
        RegisterObject(BallProjectile);

        this->particles_ = nullptr;
        this->setDestroyAfterCollision(false); //I want the ball to bounce, not to be destroyed

        //setEffect("Orxonox/sparks2");
    }

    BallProjectile::~BallProjectile()
    {
        if (this->isInitialized() && this->particles_)
        {
            this->detachOgreObject(this->particles_->getParticleSystem());
            delete this->particles_;
        }
    }

    void BallProjectile::Bounce(WorldEntity* otherObject, btManifoldPoint& contactPoint, const btCollisionShape* cs) {

        Vector3 velocity = this->getVelocity();
        Vector3 positionOtherObject = otherObject->getPosition();
        Vector3 contactPosition = this->getPosition();

        if (positionOtherObject.y < 0) {
            this->destroy();
        }
        else {
            int distance_X = positionOtherObject.x - contactPosition.x;

            if (distance_X < 0)
                distance_X = -distance_X;

            int distance_Y = positionOtherObject.y - contactPosition.y;    

            if (distance_Y < 0)
                distance_Y = -distance_Y;

            if (distance_X < distance_Y)
                velocity.x = -velocity.x;
            if (distance_Y < distance_X)
                velocity.y = -velocity.y;
            else {
                velocity.x = -velocity.x;
                velocity.y = -velocity.y;
            }
        }
    }


/**
    @brief
        The function called when a projectile hits another thing.
        Calls the hit-function, starts the shield recharge countdown, displays visual hit effects defined in Pawn.
        Needs to be called in the collidesAgainst() function by every Class directly inheriting from BasicProjectile.
    @param otherObject
        A pointer to the object the Projectile has collided against.
    @param contactPoint
        A btManifoldPoint indicating the point of contact/impact.
    @param cs
        The btCollisionShape of the other object
    @return
        Returns true if the collision resulted in a successful hit.
    @see Pawn.h
    */


    bool BallProjectile::processCollision(WorldEntity* otherObject, btManifoldPoint& contactPoint, const btCollisionShape* cs)
    {
        if (GameMode::isMaster())
        {

            Pawn* victim = orxonox_cast<Pawn*>(otherObject); // If otherObject isn't a Pawn, then victim is nullptr

            WorldEntity* entity = orxonox_cast<WorldEntity*>(this);
            assert(entity); // The projectile must not be a WorldEntity.

            // If visual effects after destruction cause problems, put this block below the effects code block
            if (victim)
            {
                victim->hit(this->getShooter(), contactPoint, cs, this->getDamage(), this->getHealthDamage(), this->getShieldDamage());
                victim->startShieldRechargeCountdown();
            }

            // Visual effects for being hit, depending on whether the shield is hit or not
            if (this->getShooter()) // If the owner does not exist (anymore?), no effects are displayed.
            {
                // Damping and explosion effect is only played if the victim is no Pawn (see cast above)
                // or if the victim is a Pawn, has no shield left, is still alive and any damage goes to the health
                if (!victim || (victim && !victim->hasShield() && victim->getHealth() > 0.0f && (this->getDamage() > 0.0f || this->getHealthDamage() > 0.0f)))
                {
                    {
                        ParticleSpawner* effect = new ParticleSpawner(this->getShooter()->getContext());
                        effect->setPosition(entity->getPosition());
                        effect->setOrientation(entity->getOrientation());
                        effect->setDestroyAfterLife(true);
                        effect->setSource("Orxonox/explosion3");
                        effect->setLifetime(2.0f);
                    }
                    // Second effect with same condition
                    {
                        ParticleSpawner* effect = new ParticleSpawner(this->getShooter()->getContext());
                        effect->setPosition(entity->getPosition());
                        effect->setOrientation(entity->getOrientation());
                        effect->setDestroyAfterLife(true);
                        effect->setSource("Orxonox/smoke4");
                        effect->setLifetime(3.0f);
                    }
                }

                // victim->isAlive() is not false until the next tick, so getHealth() > 0 is used instead
                if (victim && victim->hasShield() && (this->getDamage() > 0.0f || this->getShieldDamage() > 0.0f) && victim->getHealth() > 0.0f)
                {
                    ParticleSpawner* effect = new ParticleSpawner(this->getShooter()->getContext());
                    effect->setDestroyAfterLife(true);
                    effect->setSource("Orxonox/Shield");
                    effect->setLifetime(0.5f);
                    victim->attach(effect);
                }
            }

            Bounce(otherObject, contactPoint, cs);

            return true;
        }
        return false;
    }
}
